<?php

namespace Drupal\Tests\twig_placeholders\Unit\Twig;

use Drupal\twig_placeholders\Twig\ImageExtension;
use PHPUnit\Framework\TestCase;
use Twig\TwigFunction;
use Twig\Environment;
use Twig\Loader\ArrayLoader;

/**
 * @coversDefaultClass \Drupal\twig_placeholders\Twig\ImageExtension
 *
 * @group twig_placeholders
 */
class ImageExtensionTest extends TestCase {

  /**
   * The Twig environment.
   *
   * @var \Twig\Environment
   */
  protected Environment $twig;

  /**
   * The ImageExtension instance.
   *
   * @var \Drupal\twig_placeholders\Twig\ImageExtension
   */
  protected ImageExtension $extension;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create the Twig environment with autoescape disabled.
    $this->twig = new Environment(new ArrayLoader(), [
      // Disable autoescaping to correctly test raw HTML output.
      'autoescape' => FALSE,
    ]);

    // Create an instance of ImageExtension.
    $this->extension = new ImageExtension();

    // Register the extension in Twig.
    $this->twig->addExtension($this->extension);
  }

  /**
   * Tests that the tp_image function is registered.
   */
  public function testFunctionRegistration(): void {
    $functions = $this->extension->getFunctions();
    $functionNames = array_map(fn(TwigFunction $func) => $func->getName(), $functions);

    $this->assertContains('tp_image', $functionNames, 'The tp_image function is registered.');
  }

  /**
   * Tests the tp_image function with default parameters.
   */
  public function testDefaultImageGeneration(): void {
    $output = $this->twig->createTemplate('{{ tp_image() }}')->render();
    $this->assertStringContainsString('<img ', $output, 'Default image tag is generated.');
  }

  /**
   * Tests tp_image with URL-only mode.
   */
  public function testImageUrlOnly(): void {
    $output = $this->twig->createTemplate('{{ tp_image(true) }}')->render();
    $this->assertStringContainsString('https://picsum.photos/', $output, 'Image URL is generated.');
    $this->assertDoesNotMatchRegularExpression('/<img/', $output, 'No HTML tag is present.');
  }

  /**
   * Tests tp_image with custom width and height.
   */
  public function testCustomDimensions(): void {
    $output = $this->twig->createTemplate('{{ tp_image(false, 600, 300) }}')->render();
    $this->assertStringContainsString('https://picsum.photos/600/300', $output, 'Custom width and height are applied.');
  }

  /**
   * Tests grayscale option.
   */
  public function testGrayscale(): void {
    $output = $this->twig->createTemplate('{{ tp_image(false, 800, 450, null, true) }}')->render();
    $this->assertStringContainsString('?grayscale', $output, 'Grayscale is applied.');
  }

  /**
   * Tests blur effect.
   */
  public function testBlurEffect(): void {
    $output = $this->twig->createTemplate('{{ tp_image(false, 800, 450, null, false, 5) }}')->render();
    $this->assertStringContainsString('?blur=5', $output, 'Blur effect is applied.');
  }

  /**
   * Tests specific image ID.
   */
  public function testSpecificImageId(): void {
    $output = $this->twig->createTemplate('{{ tp_image(false, 800, 450, 10) }}')->render();
    $this->assertStringContainsString('/id/10/', $output, 'Specific image ID is applied.');
  }

  /**
   * Tests image extension selection.
   */
  public function testImageExtension(): void {
    $output = $this->twig->createTemplate('{{ tp_image(false, 800, 450, null, false, null, "webp") }}')->render();
    $this->assertStringContainsString('.webp', $output, 'WebP format is used.');
  }

}
