<?php

namespace Drupal\Tests\twig_placeholders\Unit\Twig;

use Drupal\twig_placeholders\Twig\VideoExtension;
use Drupal\Core\Messenger\MessengerInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Twig\Environment;
use Twig\Loader\ArrayLoader;
use Twig\TwigFunction;

/**
 * @coversDefaultClass \Drupal\twig_placeholders\Twig\VideoExtension
 *
 * @group twig_placeholders
 */
class VideoExtensionTest extends TestCase {

  /**
   * The Twig environment.
   *
   * @var \Twig\Environment
   */
  protected Environment $twig;

  /**
   * The VideoExtension instance.
   *
   * @var \Drupal\twig_placeholders\Twig\VideoExtension
   */
  protected VideoExtension $extension;

  /**
   * The mock MessengerInterface.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject
   */
  protected MockObject $messenger;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    /** @var \Drupal\Core\Messenger\MessengerInterface&\PHPUnit\Framework\MockObject\MockObject $messengerMock */
    $messengerMock = $this->createMock(MessengerInterface::class);
    // Explicitly a PHPUnit mock object.
    $this->messenger = $messengerMock;

    // Create the Twig environment.
    $this->twig = new Environment(new ArrayLoader(), [
      // Disable autoescaping to correctly test raw HTML output.
      'autoescape' => FALSE,
    ]);

    // Create an instance of VideoExtension.
    $this->extension = new VideoExtension($this->messenger);

    // Register the extension in Twig.
    $this->twig->addExtension($this->extension);
  }

  /**
   * Tests that the tp_video function is registered.
   */
  public function testFunctionRegistration(): void {
    $functions = $this->extension->getFunctions();
    $functionNames = array_map(fn(TwigFunction $func) => $func->getName(), $functions);

    $this->assertContains('tp_video', $functionNames, 'The tp_video function is registered.');
  }

  /**
   * Tests the tp_video function with default parameters.
   */
  public function testDefaultVideoGeneration(): void {
    // Pass a fixed video ID in the template for consistent results.
    $output = $this->twig->createTemplate('{{ tp_video(false, "Big_Buck_Bunny") }}')->render();

    // Normalize and assert that the correct video source is included.
    $normalized_output = preg_replace('/\s+/', ' ', trim($output)) ?? '';
    $this->assertStringContainsString('<source src="https://test-videos.co.uk/vids/bigbuckbunny/mp4/h264/1080/Big_Buck_Bunny_1080_10s_1MB.mp4"', $normalized_output, 'Video source is correct.');
  }

  /**
   * Tests tp_video with URL-only mode.
   */
  public function testVideoUrlOnly(): void {
    $output = $this->twig->createTemplate('{{ tp_video(true) }}')->render();
    $this->assertStringContainsString('https://test-videos.co.uk/', $output, 'Video URL is generated.');
    $this->assertDoesNotMatchRegularExpression('/<video/', $output, 'No HTML tag is present.');
  }

  /**
   * Tests tp_video with custom size.
   */
  public function testCustomVideoSize(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, "720") }}')->render();
    $this->assertStringContainsString('720', $output, 'Custom video size is applied.');
  }

  /**
   * Tests invalid video ID.
   */
  public function testInvalidVideoId(): void {
    $this->messenger->expects($this->atLeastOnce())
      ->method('addWarning')
      ->with($this->logicalAnd(
        $this->stringContains('Invalid video ID'),
        $this->stringContains('provided to tp_video()'),
        $this->stringContains('Falling back to a random video'),
        $this->stringContains('Allowed IDs:')
      ));

    // Generate the video with an invalid ID.
    $output = $this->twig->createTemplate('{{ tp_video(false, "invalid_id") }}')->render();

    // Assert that the output contains a fallback video URL.
    // Check that the URL contains a valid video ID from the allowed list.
    $valid_video_ids = VideoExtension::ALLOWED_VIDEO_IDS;
    $fallback_video_url = 'https://test-videos.co.uk/vids/';

    // Ensure the fallback URL contains one of the valid video IDs.
    $contains_valid_video_id = FALSE;

    foreach ($valid_video_ids as $valid_video_id) {
      if (strpos($output, $fallback_video_url . strtolower(str_replace('_', '', $valid_video_id))) !== FALSE) {
        $contains_valid_video_id = TRUE;
        break;
      }
    }

    // Assert that the output contains one of the valid video IDs in the
    // fallback URL.
    $this->assertTrue($contains_valid_video_id, 'Fallback video source contains a valid video ID.');
  }

  /**
   * Tests tp_video with valid video ID.
   */
  public function testValidVideoId(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, "Sintel") }}')->render();
    $this->assertStringContainsString('Sintel', $output, 'Valid video ID is used.');
  }

  /**
   * Tests tp_video with custom extension.
   */
  public function testVideoExtension(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, "webm") }}')->render();
    $this->assertStringContainsString('.webm', $output, 'WebM extension is used.');
  }

  /**
   * Tests autoplay option for tp_video.
   */
  public function testAutoplayOption(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, null, true) }}')->render();
    $this->assertStringContainsString('autoplay', $output, 'Autoplay is applied.');
  }

  /**
   * Tests controls option for tp_video.
   */
  public function testControlsOption(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, null, false, true) }}')->render();
    $this->assertStringContainsString('controls', $output, 'Controls are applied.');
  }

  /**
   * Tests loop option for tp_video.
   */
  public function testLoopOption(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, null, false, false, true) }}')->render();
    $this->assertStringContainsString('loop', $output, 'Loop is applied.');
  }

  /**
   * Tests muted option for tp_video.
   */
  public function testMutedOption(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, null, false, false, false, true) }}')->render();
    $this->assertStringContainsString('muted', $output, 'Muted is applied.');
  }

  /**
   * Tests playsinline option for tp_video.
   */
  public function testPlaysInlineOption(): void {
    $output = $this->twig->createTemplate('{{ tp_video(false, null, null, null, false, false, false, false, true) }}')->render();
    $this->assertStringContainsString('playsinline', $output, 'Playsinline is applied.');
  }

}
